@@ -43,7 +43,6 @@ uiFun <- function(beta, Y, X, delta, clsize, sigma, n, Z, weights, smooth = TRUE
   p <- ncol(X)
   ans <- numeric(p)
   sn <- vector("double", p)
-  ## ans <- .C("unsfun", as.double(beta), as.double(Y), as.double(X), as.double(delta), as.integer(clsize), as.double(sigma), as.integer(n), as.integer(p), as.integer(N), as.double(Z), as.double(weights), out = as.double(sn), PACKAGE = "aftgee")$out
   ans <- .C("ufun", as.double(beta), as.double(Y), as.double(X), as.double(delta), as.integer(clsize), as.double(sigma), as.integer(n), as.integer(p), as.integer(N), as.double(Z), as.double(weights), out = as.double(sn), PACKAGE = "aftgee")$out
   ans <- ans - constant
 }
@@ -85,8 +84,6 @@ uilogFun <- function(beta, Y, X, delta, clsize, sigma, n, Z, weights, smooth = T
 viEmp <- function(beta, Y, delta, X, id, weights = rep(1, nrow(X)), B = 500, mb = TRUE, zbeta = FALSE, smooth = TRUE, rankweights = "gehan", sigma = diag(ncol(X))){
   p <- ncol(X)
   clsize <- unlist(lapply(split(id, id), length))
-  ## n <- length(clsize)
-  ## N <- sum(clsize)
   n <- sum(unlist(lapply(split(weights, id), unique)))
   N <- sum(weights)
   UnV <- zmat <- matrix(0, ncol = B, nrow = p)
@@ -112,7 +109,6 @@ viEmp <- function(beta, Y, delta, X, id, weights = rep(1, nrow(X)), B = 500, mb
         }
         if (rankweights != "gehan") {
             UnV[,i] <- uilogFun(newbeta, Y, X, delta, clsize, sigma, length(clsize), Z, weights, smooth, constant = 0, rankweights) # / n
-            ## as.vector(.C("ulogfun", as.double(newbeta), as.double(Y), as.double(X), as.double(delta), as.integer(clsize), as.double(sigma), as.integer(n), as.integer(p), as.integer(N), as.double(Z), as.double(weights), out = as.double(sn), PACKAGE = "aftgee")$out) / n
         }
     }
     if (smooth == FALSE) {
@@ -121,7 +117,6 @@ viEmp <- function(beta, Y, delta, X, id, weights = rep(1, nrow(X)), B = 500, mb
         }
         if (rankweights != "gehan") {
             UnV[,i] <- uilogFun(newbeta, Y, X, delta, clsize, sigma, length(clsize), Z, weights, smooth, constant = 0, rankweights) # / n
-            ## as.vector(.C("ulognsfun", as.double(newbeta), as.double(Y), as.double(X), as.double(delta), as.integer(clsize), as.double(sigma), as.integer(n), as.integer(p), as.integer(N), as.double(Z), as.double(weights), out = as.double(sn), PACKAGE = "aftgee")$out) / n
         }
     }
 }
@@ -146,16 +141,12 @@ get_si <- function(beta, Y, delta, X, id, weights = rep(1, nrow(X)), B = 500, ra
     Shati <- survfit(Surv(ei, deltai) ~ 1, weights = weightsi)$surv
     Shati <- rep(Shati, repeats)
     Shat[dummy[ord]] <- Shati
-    ## Shati <- as.matrix(survfit(Surv(ei, deltai) ~ 1, weights = weightsi, etype = withinIdi)$surv)
-    ## Shati <- Shati[rep(1:nrow(Shati), repeats), ]
-    ## Shat <- Shati[order(dummy[ord]),]
     xdif <- X[ik,] - X[jl,]
     edif <- en[ik] - en[jl]
     ind <- ifelse(edif <= 0, 1, 0)
     minEn <- ifelse(edif <= 0, ik, jl)
     si <- s <- NULL
     if (rankweights == "gehan") {
-        ## s <- weights[ik] * weights[jl] * delta[ik] * ind * xdif + weights[ik] * weights[jl] * log(Shat[minEn]) * xdif
         s <- weights[jl] * delta[ik] * ind * xdif + weights[jl] * log(Shat[minEn]) * xdif
         if (length(which(s == Inf)) > 0) {
             s <- ifelse(s == Inf, 0, s)
@@ -166,15 +157,13 @@ get_si <- function(beta, Y, delta, X, id, weights = rep(1, nrow(X)), B = 500, ra
         if (sum(is.na(s) > 0)) {
             s <- ifelse(is.na(s) == TRUE, 0, s)
         }
-        s <- rowsum(s, ik)  ## N X p
+        s <- rowsum(s, ik)
         }
     if (rankweights == "logrank") {
         haz <- -1 * log(Shat)
         haz <- ifelse(haz == Inf, 0, haz)
         haz <- ifelse(haz == -Inf, 0, haz)
         haz <- ifelse(is.na(haz) == TRUE, 0, haz)
-        ## gamma1 <- rowsum(ind * X[jl,] * weights[ik] * weights[jl], ik)
-        ## gamma0 <- as.numeric(rowsum(ind * weights[ik] * weights[jl], ik))
         gamma1 <- rowsum(ind * X[jl,] * weights[jl], ik)
         gamma0 <- as.numeric(rowsum(ind * weights[jl], ik))
         si1i <-  si1 <- s2 <- matrix(0, nrow = N, ncol = p)
@@ -185,15 +174,12 @@ get_si <- function(beta, Y, delta, X, id, weights = rep(1, nrow(X)), B = 500, ra
         s2[dummy[ord],] <- s2s
         s <- delta[1:N] * si1 - s2
     }
-    ## s <- subset(s, given)
     s
 }
 
 get_vi <- function(s, id, delta, weights, n) {
-    ## clsize <- unlist(lapply(split(id, id), length))
-    ## nstra <- length(unique(weights))
     clweights <- as.numeric(unlist(lapply(split(weights, id), unique)))
-    s1 <- rowsum(s, group = id) ## / 2 ##
+    s1 <- rowsum(s, group = id)
     si1 <- lapply(split(s1, 1:nrow(s1)), function(x) x %o% x)
     s11 <- mapply("*", si1, clweights, SIMPLIFY = FALSE)
     v1 <- Reduce("+", s11) / n
@@ -208,9 +194,6 @@ get_vi <- function(s, id, delta, weights, n) {
         cweights <- cweights[cweights != 1 & cweights != 0]
         vi <- (cweights - 1)* (v1 - v2)
     }
-    ## v1 <- v1 / n
-    ## v2 <- v2 / n
-    ## vi <- vi / n
     list(vi = vi, v1 = v1, v2 = v2)
 }
 
@@ -218,7 +201,6 @@ get_vi <- function(s, id, delta, weights, n) {
 
 viClo <- function(beta, Y, delta, X, id, weights = rep(1, nrow(X)), B = 500, rankweights = "gehan", stratify = TRUE) {
     s <- v1 <- vi <- v2i <- NULL
-    ## n <- length(unlist(lapply(split(weights, id), unique)))
     n <- sum(unlist(lapply(split(weights, id), unique)))
     p <- ncol(X)
     clsize <- unlist(lapply(split(id, id), length))
@@ -228,7 +210,7 @@ viClo <- function(beta, Y, delta, X, id, weights = rep(1, nrow(X)), B = 500, ran
     s <- get_si(beta, Y, delta, X, id, weights, B, rankweights)
     if (stratify == TRUE) {
         v1 <- get_vi(s, id, delta, weights, n)$v1
-        vi <- v1 ## / n
+        vi <- v1
         v2 <- matrix(0, ncol = p, nrow = p)
         if (length(unique(stra)) > 1) {
             for (i in 1:length(unique(stra))) {
@@ -238,11 +220,9 @@ viClo <- function(beta, Y, delta, X, id, weights = rep(1, nrow(X)), B = 500, ran
                               weights = weights[stra == i],
                               n = ns
                               )$vi
-                ## strPr <- sum(unlist(lapply(split(weights[stra == i], id[stra == i]), unique))) / N
                 strPr <- ns / n
                 v2 <- v2 + v2i * strPr
             }
-            ## vi <- (v1 + v2) / n
             vi <- v1 + v2
         }
     }
@@ -400,9 +380,9 @@ smoothrr <- function(formula, data, subset, contrasts = NULL, id,
   if (is.null(weights)) weights <- rep(1, N)
   stratify <- TRUE
   if (is.null(strataid)) stratify <- FALSE
-  xnames <- colnames(x) ## [-1]
-  if(is.null(sigmainit)) {sigmainit = diag(ncol(x))} ## x[,-1]
-  out <- smoothrr.fit(Y = log(y[,1]), delta = y[,2], X = as.matrix(x), id = id, weights = weights, binit = binit, sigmainit = sigmainit, variance = variance, B = B, rankweights = rankweights, stratify = stratify, control = control) ## x[,-1]
+  xnames <- colnames(x)
+  if(is.null(sigmainit)) {sigmainit = diag(ncol(x))}
+  out <- smoothrr.fit(Y = log(y[,1]), delta = y[,2], X = as.matrix(x), id = id, weights = weights, binit = binit, sigmainit = sigmainit, variance = variance, B = B, rankweights = rankweights, stratify = stratify, control = control)
   out$call <- scall
   out$vari.name <- xnames
   class(out) <- "smoothrr"
@@ -415,188 +395,178 @@ smoothrr.fit <- function(Y, delta, X, binit = "lm",
                          variance = c("MB", "ZLCF", "ZLMB", "sHCF", "sHMB", "ISCF", "ISMB", "js"),
                          B = 100, rankweights = "gehan", stratify = TRUE,
                          control = aftgee.control()) {
-  p <- ncol(X)
-  go <- 1
-  step <- 0
-  if (is.numeric(binit)) {
-      if (length(binit) != p) {
-          stop("Binit value length does not match with numbers of covariates.")
-      }
-      beta <- binit
-  }
-  if (!(is.numeric(binit))) {
-      if (binit == "lm") {
-          beta <- lm(Y ~ X - 1)$coef
-      }
-  }
-  beta.temp1 <- beta
-  sigmainit.temp1 <- sigmainit
-  beta.temp2 <- NULL
-  sigmainit.temp2 <- NULL
-  clsize <- unlist(lapply(split(id, id), length))
-  order <- unlist(lapply(clsize, function(x) 1:x))
-  N <- sum(clsize)
-  n <- length(clsize)
-  Z <- rep(1, N)
-  var.MB <- var.ZLCF <- var.ZLMB <- var.sHCF <- var.sHMB <- var.js <- var.ISCF <- var.ISMB <- NaN
-  for (i in 1:control$maxiter) {
-      ## Point Estimation
-      if (rankweights %in% c("gehan", "PW", "Prentice-Wilcoxon", "GP") & i == 1) {
-          tbeta <- system.time(beta.temp2 <- nlm(Ln, p = beta.temp1, other = list(Y, X, delta, clsize, sigmainit.temp1, weights, Z), fscale = 0.01)$estimate)
-          ## tbeta <- system.time(temp <- BBsolve(beta.temp1, uiFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = sigmainit.temp1, n = n, Z = Z, weights = weights, smooth = TRUE, constant = 0, quiet = TRUE))
-          ## beta.temp2 <- temp$par
-          ## beta.conv <- temp$convergence
-          beta.conv <- 0
-      }
-      if (rankweights == "logrank") {
-          tbeta <- system.time(temp <- BBsolve(beta.temp1, uilogFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = sigmainit.temp1, n = n, Z = Z, weights = weights, smooth = TRUE, constant = 0, rankweights = rankweights, quiet = TRUE))
-          beta.temp2 <- temp$par
-          beta.conv <- temp$convergence
-      }
-      if (rankweights == "PW" | rankweights == "Prentice-Wilcoxon") {
-          beta.temp1 <- beta.temp2
-          tbeta <- system.time(temp <- BBsolve(beta.temp1, uilogFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = sigmainit.temp1, n = n, Z = Z, weights = weights, smooth = TRUE, constant = 0, rankweights = rankweights, quiet = TRUE))
-          beta.temp2 <- temp$par
-          print(beta.temp1)
-
-          ## if (abs(beta.temp1 - beta.temp2) < control$abstol) {break}
-          if (max(abs(beta.temp1 - beta.temp2)) >= control$abstol) {
-              beta.temp1 <- beta.temp2
-              next
-          }
-          beta.conv <- ifelse(i == control$maxiter, 1, 0)
-      }
-      if (rankweights == "GP") {
-          beta.temp1 <- beta.temp2
-          tbeta <- system.time(temp <- BBsolve(beta.temp1, uilogFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = sigmainit.temp1, n = n, Z = Z, weights = weights, smooth = TRUE, constant = 0, rankweights = rankweights, quiet = TRUE))
-          beta.temp2 <- temp$par
-          print(beta.temp1)
-
-          ## if (abs(beta.temp1 - beta.temp2) < control$abstol) {break}
-          if (max(abs(beta.temp1 - beta.temp2)) >= control$abstol) {
-              beta.temp1 <- beta.temp2
-              next
-          }
-          beta.conv <- ifelse(i == control$maxiter, 1, 0)
-      }
-      ## Ends Point Estimation
-
-      pass.MB <- pass.ZLCF <- pass.ZLMB <- pass.sHCF <- pass.sHMB <- pass.ISCF <- pass.ISMB <- FALSE
-      ZLMB.An.inv <- ZLCF.An.inv <- ISMB.An.inv <- ISCF.An.inv <- js.An.inv <- 1
-      if (B == 0) {break}
-      ## begin multipler resampling method
-      if (sum(variance %in% "MB") > 0 && pass.MB == FALSE) {
-          temp <- matrix(0, nrow = B, ncol = p)
-          if(B == 0) {
-              var.MB <- NULL
-              pass.MB <- TRUE
-              if (sum(c("ZLCF", "js", "ZLMB", "sHCF", "sHMB", "ISCF", "ISMB") %in% variance) == 0) {break}
-          }
-          for (i in 1:B) {
-              Z <- rep(rexp(length(clsize)), clsize)
-
-              if (rankweights == "gehan") {
-                  temp[i,] <- nlm(Ln, p= beta.temp2, other = list(Y, X, delta, clsize, sigmainit.temp1, weights, Z), fscale = 0.01)$estimate
-              }
-              if (rankweights == "logrank") {
-                  temp[i,] <- BBsolve(beta.temp2, uilogFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = sigmainit.temp1, n = n, Z = Z, weights = weights, smooth = TRUE, constant = 0, rankweights = rankweights, quiet = TRUE)$par
-              }
-              if (rankweights == "PW" | rankweights == "Prentice-Wilcoxon") {
-                  temp[i,] <- BBsolve(beta.temp2, uilogFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = sigmainit.temp1, n = n, Z = Z, weights = weights, smooth = TRUE, constant = 0, rankweights = rankweights, quiet = TRUE)$par
-              }
-              if (rankweights == "GP") {
-                  temp[i,] <- BBsolve(beta.temp2, uilogFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = sigmainit.temp1, n = n, Z = Z, weights = weights, smooth = TRUE, constant = 0, rankweights = rankweights, quiet = TRUE)$par
-              }
-          }
-          var.MB <- var(temp)
-          pass.MB <- TRUE
-          if (sum(c("ZLCF", "js", "ZLMB", "sHCF", "sHMB", "ISCF", "ISMB") %in% variance) == 0) {break}
-      } ## end MBipler resampling method
-
-      ## begin Zeng and Lin's approach, close Si
-      if (sum(variance %in% "ZLCF") > 0 && pass.ZLCF == FALSE) {
-          var.ZLCF <- zlFun(beta.temp2, Y, delta, X, id, weights, B, vClose = TRUE, rankweights, stratify = stratify, sigma = sigmainit.temp1)
-          ## if (var.ZLCF$An.inv == 0) {print (var.ZLCF$An.msg)}
-          ZLCF.An.inv <- var.ZLCF$An.inv
-          var.ZLCF <- var.ZLCF$covmat
-          pass.ZLCF <- TRUE
-          if (sum(c("js", "ZLMB", "sHCF", "sHMB", "ISCF", "ISMB") %in% variance) == 0) {break}
-      } ## end Zeng and Lin's approach, close Si
-
-      ## begin Zeng and Lin's approach, emprical Si
-      if (sum(variance %in% "ZLMB") > 0 && pass.ZLMB == FALSE) {
-          var.ZLMB <- zlFun(beta.temp2, Y, delta, X, id, weights, B, vClose = FALSE, rankweights, sigma = sigmainit.temp1)
-          ## if(var.ZLMB$An.inv) {print(var.ZLMB$An.msg)}
-          ZLMB.An.inv <- var.ZLMB$An.inv
-          var.ZLMB <- var.ZLMB$covmat
-          pass.ZLMB <- TRUE
-          if (sum(c("js", "sHCF", "sHMB", "ISCF", "ISMB") %in% variance) == 0) {break}
-      } ## end Zeng and Lin's approach, emprical Si
-
-    ## begin Huang's approach, close Si
-    if (sum(variance %in% "sHCF") > 0 && pass.sHCF == FALSE){
-      var.sHCF <- huangFun(beta.temp2, Y, delta, X, id, weights, B, vClose = TRUE, rankweights, sigma = sigmainit.temp1, stratify = stratify)$covmat
-      pass.sHCF <- TRUE
-      if (sum(c("js", "sHMB", "ISCF", "ISMB") %in% variance) == 0) {break}
-    } ## end Huang's approach, close Si
-
-    ## begin Huang's approach, emprical Si
-    if (sum(variance %in% "sHMB") > 0 && pass.sHMB == FALSE){
-      var.sHMB <- huangFun(beta.temp2, Y, delta, X, id, weights, B, vClose = FALSE, rankweights, sigma = sigmainit.temp1)$covmat
-      pass.sHMB <- TRUE
-      if (sum(c("js", "ISCF", "ISMB") %in% variance) == 0) {break}
-    } ## end Huang's approach, emprical Si
-
-    ## begin IS's appraoch, close Si
-    if (sum(variance %in% "ISCF") > 0 && pass.ISCF == FALSE) {
-        var.ISCF <- isFun(beta.temp2, Y, delta, X, id, weights, sigmainit.temp1, B, vClose = TRUE, rankweights, stratify = stratify)
-        ## if(var.ISCF$An.inv == 0){print(var.ISCF$An.msg)}
-        ISCF.An.inv <- var.ISCF$An.inv
-        var.ISCF <- var.ISCF$covmat
-        if (sum(c("js", "ISMB") %in% variance) == 0) {break}
-        pass.ISCF <- TRUE
+    p <- ncol(X)
+    go <- 1
+    step <- 0
+    if (is.numeric(binit)) {
+        if (length(binit) != p) {
+            stop("Binit value length does not match with numbers of covariates.")
+        }
+        beta <- binit
     }
-    ## end IS's approach, close Si
-
-    ## begin IS's appraoch, emprical Si
-    if (sum(variance %in% "ISMB") > 0 && pass.ISMB == FALSE) {
-        var.ISMB <- isFun(beta.temp2, Y, delta, X, id, weights, sigmainit.temp1, B, vClose = FALSE, rankweights)
-        ## if(var.ISMB$An.inv == 0) {print(var.ISMB$An.msg)}
-        ISMB.An.inv <- var.ISMB$An.inv
-        var.ISMB <- var.ISMB$covmat
-        if (!("js" %in% variance)) {break}
-        pass.ISMB <- TRUE
+    if (!(is.numeric(binit))) {
+        if (binit == "lm") {
+            beta <- lm(Y ~ X - 1)$coef
+        }
     }
-    ## end IS's approach, emprical Si
+    beta.temp1 <- beta
+    sigmainit.temp1 <- sigmainit
+    beta.temp2 <- NULL
+    sigmainit.temp2 <- NULL
+    clsize <- unlist(lapply(split(id, id), length))
+    order <- unlist(lapply(clsize, function(x) 1:x))
+    N <- sum(clsize)
+    n <- length(clsize)
+    Z <- rep(1, N)
+    var.MB <- var.ZLCF <- var.ZLMB <- var.sHCF <- var.sHMB <- var.js <- var.ISCF <- var.ISMB <- NaN
+    for (i in 1:control$maxiter) {
+        ## Point Estimation
+        if (rankweights %in% c("gehan", "PW", "Prentice-Wilcoxon", "GP") & i == 1) {
+            tbeta <- system.time(beta.temp2 <- nlm(Ln, p = beta.temp1, other = list(Y, X, delta, clsize, sigmainit.temp1, weights, Z), fscale = 0.01)$estimate)
+            ## tbeta <- system.time(temp <- BBsolve(beta.temp1, uiFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = sigmainit.temp1, n = n, Z = Z, weights = weights, smooth = TRUE, constant = 0, quiet = TRUE))
+            ## beta.temp2 <- temp$par
+            ## beta.conv <- temp$convergence
+            beta.conv <- 0
+        }
+        if (rankweights == "logrank") {
+            tbeta <- system.time(temp <- BBsolve(beta.temp1, uilogFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = sigmainit.temp1, n = n, Z = Z, weights = weights, smooth = TRUE, constant = 0, rankweights = rankweights, quiet = TRUE))
+            beta.temp2 <- temp$par
+            beta.conv <- temp$convergence
+        }
+        if (rankweights == "PW" | rankweights == "Prentice-Wilcoxon") {
+            beta.temp1 <- beta.temp2
+            tbeta <- system.time(temp <- BBsolve(beta.temp1, uilogFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = sigmainit.temp1, n = n, Z = Z, weights = weights, smooth = TRUE, constant = 0, rankweights = rankweights, quiet = TRUE))
+            beta.temp2 <- temp$par
+            print(beta.temp1)
 
-    ## begin JS's iterative approach
-    if (sum(variance %in% "js") > 0) {
-      var.js <- isFun(beta.temp2, Y, delta, X, id, weights, sigmainit.temp1, omega = TRUE, rankweights = rankweights, stratify = stratify)
-      ## if(var.js$An.inv == 0) {print(var.js$An.msg)}
-      js.An.inv <- var.js$An.inv
-      sigmainit.temp2 <- var.js$covmat
-      e.sigma <- abs(sigmainit.temp1-sigmainit.temp2)
-      e.beta <- abs(beta.temp1 - beta.temp2)
-      e.rel <- max(max(e.sigma/abs(sigmainit.temp2)), max(e.beta/abs(beta.temp2)))
-      e.abs <- max(e.sigma, e.beta)
-      if (abs(e.rel) < control$reltol) break
-      if (abs(e.abs) < control$abstol) break
-      ## mat add an absolute tolerance check
-      beta.temp1 <- beta.temp2
-      sigmainit.temp1 <- sigmainit.temp2
-      ## var.js <- sigmainit.temp2 / n
-    } ## end js
-  }
-  covmat <- list(MB = var.MB, ZLCF = var.ZLCF, ZLMB = var.ZLMB, sHCF = var.sHCF, sHMB = var.sHMB, ISCF = var.ISCF, ISMB = var.ISMB, js = sigmainit.temp2 / n)
-  convergence <- if ( i == control$maxiter) 1 else 0
-  out <- list(beta = beta.temp2, covmat = covmat, convergence = convergence, tbeta = tbeta, beta.conv = beta.conv, beta.conv.at = i, var.meth = variance,
-              ZLMB.An.inv = ZLMB.An.inv, ZLCF.An.inv = ZLCF.An.inv, ISMB.An.inv = ISMB.An.inv, ISCF.An.inv = ISCF.An.inv, js.An.inv = js.An.inv)
-  ## if (ZLMB.An.inv == 0 | ZLCF.An.inv == 0 | ISMB.An.inv == 0 | ISCF.An.inv == 0 | js.An.inv == 0) {
-  ##      print("An is singular")
-  ## }
-  class(out) <- "smoothrr.fit"
-  return(out)
+            if (max(abs(beta.temp1 - beta.temp2)) >= control$abstol) {
+                beta.temp1 <- beta.temp2
+                next
+            }
+            beta.conv <- ifelse(i == control$maxiter, 1, 0)
+        }
+        if (rankweights == "GP") {
+            beta.temp1 <- beta.temp2
+            tbeta <- system.time(temp <- BBsolve(beta.temp1, uilogFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = sigmainit.temp1, n = n, Z = Z, weights = weights, smooth = TRUE, constant = 0, rankweights = rankweights, quiet = TRUE))
+            beta.temp2 <- temp$par
+            print(beta.temp1)
+
+            if (max(abs(beta.temp1 - beta.temp2)) >= control$abstol) {
+                beta.temp1 <- beta.temp2
+                next
+            }
+            beta.conv <- ifelse(i == control$maxiter, 1, 0)
+        }
+        ## Ends Point Estimation
+        beta.MB <- 0
+        pass.MB <- pass.ZLCF <- pass.ZLMB <- pass.sHCF <- pass.sHMB <- pass.ISCF <- pass.ISMB <- FALSE
+        ZLMB.An.inv <- ZLCF.An.inv <- ISMB.An.inv <- ISCF.An.inv <- js.An.inv <- 1
+        if (B == 0) {break}
+        ## begin MB resampling method
+        if (sum(variance %in% "MB") > 0 && pass.MB == FALSE) {
+            temp <- matrix(0, nrow = B, ncol = p)
+            if(B == 0) {
+                var.MB <- NULL
+                pass.MB <- TRUE
+                if (sum(c("ZLCF", "js", "ZLMB", "sHCF", "sHMB", "ISCF", "ISMB") %in% variance) == 0) {break}
+            }
+            for (i in 1:B) {
+                Z <- rep(rexp(length(clsize)), clsize)
+                if (rankweights == "gehan") {
+                    temp[i,] <- nlm(Ln, p= beta.temp2, other = list(Y, X, delta, clsize, sigmainit.temp1, weights, Z), fscale = 0.01)$estimate
+                }
+                if (rankweights == "logrank") {
+                    temp[i,] <- BBsolve(beta.temp2, uilogFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = sigmainit.temp1, n = n, Z = Z, weights = weights, smooth = TRUE, constant = 0, rankweights = rankweights, quiet = TRUE)$par
+                }
+                if (rankweights == "PW" | rankweights == "Prentice-Wilcoxon") {
+                    temp[i,] <- BBsolve(beta.temp2, uilogFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = sigmainit.temp1, n = n, Z = Z, weights = weights, smooth = TRUE, constant = 0, rankweights = rankweights, quiet = TRUE)$par
+                }
+                if (rankweights == "GP") {
+                    temp[i,] <- BBsolve(beta.temp2, uilogFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = sigmainit.temp1, n = n, Z = Z, weights = weights, smooth = TRUE, constant = 0, rankweights = rankweights, quiet = TRUE)$par
+                }
+            }
+            beta.MB <- temp
+            var.MB <- var(temp)
+            pass.MB <- TRUE
+            if (sum(c("ZLCF", "js", "ZLMB", "sHCF", "sHMB", "ISCF", "ISMB") %in% variance) == 0) {break}
+        } ## end MB resampling method
+
+        ## begin Zeng and Lin's approach, close Si
+        if (sum(variance %in% "ZLCF") > 0 && pass.ZLCF == FALSE) {
+            var.ZLCF <- zlFun(beta.temp2, Y, delta, X, id, weights, B, vClose = TRUE, rankweights, stratify = stratify, sigma = sigmainit.temp1)
+            ZLCF.An.inv <- var.ZLCF$An.inv
+            var.ZLCF <- var.ZLCF$covmat
+            pass.ZLCF <- TRUE
+            if (sum(c("js", "ZLMB", "sHCF", "sHMB", "ISCF", "ISMB") %in% variance) == 0) {break}
+        } ## end Zeng and Lin's approach, close Si
+
+        ## begin Zeng and Lin's approach, emprical Si
+        if (sum(variance %in% "ZLMB") > 0 && pass.ZLMB == FALSE) {
+            var.ZLMB <- zlFun(beta.temp2, Y, delta, X, id, weights, B, vClose = FALSE, rankweights, sigma = sigmainit.temp1)
+            ZLMB.An.inv <- var.ZLMB$An.inv
+            var.ZLMB <- var.ZLMB$covmat
+            pass.ZLMB <- TRUE
+            if (sum(c("js", "sHCF", "sHMB", "ISCF", "ISMB") %in% variance) == 0) {break}
+        } ## end Zeng and Lin's approach, emprical Si
+
+        ## begin Huang's approach, close Si
+        if (sum(variance %in% "sHCF") > 0 && pass.sHCF == FALSE){
+            var.sHCF <- huangFun(beta.temp2, Y, delta, X, id, weights, B, vClose = TRUE, rankweights, sigma = sigmainit.temp1, stratify = stratify)$covmat
+            pass.sHCF <- TRUE
+            if (sum(c("js", "sHMB", "ISCF", "ISMB") %in% variance) == 0) {break}
+        } ## end Huang's approach, close Si
+
+        ## begin Huang's approach, emprical Si
+        if (sum(variance %in% "sHMB") > 0 && pass.sHMB == FALSE){
+            var.sHMB <- huangFun(beta.temp2, Y, delta, X, id, weights, B, vClose = FALSE, rankweights, sigma = sigmainit.temp1)$covmat
+            pass.sHMB <- TRUE
+            if (sum(c("js", "ISCF", "ISMB") %in% variance) == 0) {break}
+        } ## end Huang's approach, emprical Si
+
+        ## begin IS's appraoch, close Si
+        if (sum(variance %in% "ISCF") > 0 && pass.ISCF == FALSE) {
+            var.ISCF <- isFun(beta.temp2, Y, delta, X, id, weights, sigmainit.temp1, B, vClose = TRUE, rankweights, stratify = stratify)
+            ISCF.An.inv <- var.ISCF$An.inv
+            var.ISCF <- var.ISCF$covmat
+            if (sum(c("js", "ISMB") %in% variance) == 0) {break}
+            pass.ISCF <- TRUE
+        }
+        ## end IS's approach, close Si
+
+        ## begin IS's appraoch, emprical Si
+        if (sum(variance %in% "ISMB") > 0 && pass.ISMB == FALSE) {
+            var.ISMB <- isFun(beta.temp2, Y, delta, X, id, weights, sigmainit.temp1, B, vClose = FALSE, rankweights)
+            ISMB.An.inv <- var.ISMB$An.inv
+            var.ISMB <- var.ISMB$covmat
+            if (!("js" %in% variance)) {break}
+            pass.ISMB <- TRUE
+        }
+        ## end IS's approach, emprical Si
+
+        ## begin JS's iterative approach
+        if (sum(variance %in% "js") > 0) {
+            var.js <- isFun(beta.temp2, Y, delta, X, id, weights, sigmainit.temp1, omega = TRUE, rankweights = rankweights, stratify = stratify)
+            ## if(var.js$An.inv == 0) {print(var.js$An.msg)}
+            js.An.inv <- var.js$An.inv
+            sigmainit.temp2 <- var.js$covmat
+            e.sigma <- abs(sigmainit.temp1-sigmainit.temp2)
+            e.beta <- abs(beta.temp1 - beta.temp2)
+            e.rel <- max(max(e.sigma/abs(sigmainit.temp2)), max(e.beta/abs(beta.temp2)))
+            e.abs <- max(e.sigma, e.beta)
+            if (abs(e.rel) < control$reltol) break
+            if (abs(e.abs) < control$abstol) break
+            beta.temp1 <- beta.temp2
+            sigmainit.temp1 <- sigmainit.temp2
+        }
+    }
+    covmat <- list(MB = var.MB, ZLCF = var.ZLCF, ZLMB = var.ZLMB, sHCF = var.sHCF, sHMB = var.sHMB, ISCF = var.ISCF, ISMB = var.ISMB, js = sigmainit.temp2 / n)
+    convergence <- if ( i == control$maxiter) 1 else 0
+    out <- list(beta = beta.temp2, covmat = covmat, convergence = convergence, tbeta = tbeta, beta.conv = beta.conv, beta.conv.at = i, betaMB = beta.MB, var.meth = variance,
+                ZLMB.An.inv = ZLMB.An.inv, ZLCF.An.inv = ZLCF.An.inv, ISMB.An.inv = ISMB.An.inv, ISCF.An.inv = ISCF.An.inv, js.An.inv = js.An.inv)
+    class(out) <- "smoothrr.fit"
+    return(out)
 }
 
 
 
+
